@@ -20,7 +20,9 @@
 #include "../../configure.h"
 #define LOCAL_DEBUG
 
+
 #include "asinternals.h"
+#include "../../libAfterConf/afterconf.h"
 
 #include <signal.h>
 #include <fcntl.h>
@@ -40,12 +42,15 @@
 ASFlagType AfterStepState = 0; /* default status */
 
 /* Config : */
-struct ASDatabase    *Database = NULL;
-
 ASVector     *Modules       = NULL;
 int           Module_fd     = 0;
 int           Module_npipes = 8;
 
+ASBalloonState *MenuBalloons = NULL ; 
+ASBalloonState *TitlebarBalloons = NULL ; 
+
+char *original_DISPLAY_string = NULL;
+
 /**************************************************************************/
 void          SetupScreen();
 void          CleanupScreen();
@@ -72,6 +77,18 @@ LOCAL_DEBUG_OUT( "pid(%d),closing x_fd...", getpid() );
 LOCAL_DEBUG_OUT( "pid(%d),complete...", getpid() );
 }
 #endif
+#ifdef DEBUG_TRACE_X
+const char *window_id2name (Display * dpy, Window w, int *how_to_free)
+{
+	ASWindow *asw = window2ASWindow( w );
+	if( asw ) 
+	{
+		*how_to_free = 1 ;
+		return mystrdup(ASWIN_NAME(asw));
+	}
+	return NULL;
+}
+#endif
 
 
 /**************************************************************************/
@@ -109,6 +126,10 @@ main (int argc, char **argv, char **envp)
 	_as_grab_screen_func = GrabEm;
 	_as_ungrab_screen_func = UngrabEm;
 
+	original_DISPLAY_string = getenv("DISPLAY");
+	if (original_DISPLAY_string)
+		original_DISPLAY_string = mystrdup(original_DISPLAY_string);
+
 #ifdef DEBUG_TRACE_X
 	trace_window_id2name_hook = &window_id2name;
 #endif
@@ -173,10 +194,6 @@ SHOW_CHECKPOINT;
 SHOW_CHECKPOINT;
 	XSync (dpy, 0);
 SHOW_CHECKPOINT;
-#ifdef XSHMIMAGE
-	enable_shmem_images ();
-SHOW_CHECKPOINT;
-#endif
     set_parent_hints_func( afterstep_parent_hints_func ); /* callback for collect_hints() */
 SHOW_CHECKPOINT;
     SetupModules();
@@ -198,7 +215,7 @@ SHOW_CHECKPOINT;
         {
             if( !get_flags(MyArgs.flags, ASS_SingleScreen) )
             {
-                int pid = spawn_child( MyName, (i<MAX_USER_SINGLETONS_NUM)?i:-1, i, None, C_NO_CONTEXT, True, True, NULL );
+                int pid = spawn_child( MyName, (i<MAX_USER_SINGLETONS_NUM)?i:-1, i, NULL, None, C_NO_CONTEXT, True, True, NULL );
                 if( pid >= 0 )
                     show_progress( "\t instance of afterstep spawned with pid %d.", pid );
                 else
@@ -259,7 +276,6 @@ SHOW_CHECKPOINT;
 
     /* Load config ... */
     /* read config file, set up menus, colors, fonts */
-    InitDatabase (False);
     LoadASConfig (0, PARSE_EVERYTHING);
 
     /* Reparent all the windows and setup pan frames : */
@@ -395,6 +411,9 @@ SetupScreen()
     Scr.Windows = init_aswindow_list();
 
 	XSelectInput (dpy, Scr.Root, AS_ROOT_EVENT_MASK);
+
+	MenuBalloons = create_balloon_state(); 
+	TitlebarBalloons = create_balloon_state(); 
 }
 
 void
@@ -408,6 +427,12 @@ CleanupScreen()
         destroy_aswindow_list( &(Scr.Windows), True );
         ungrab_server();
     }
+
+	destroy_balloon_state(&TitlebarBalloons); 
+	destroy_balloon_state(&MenuBalloons); 
+
+	release_all_old_background( True );
+
     DestroyManagementWindows();
     CleanupColormaps();
 
@@ -536,7 +561,7 @@ CaptureAllWindows (ScreenInfo *scr)
     for (i = 0; i < nchildren; i++)
         if (children[i] )
         {
-            unsigned long nitems = 0;
+            long nitems = 0;
             CARD32 *state_prop = NULL ;
             int wm_state = DontCareState ;
             int k ;
@@ -568,7 +593,7 @@ CaptureAllWindows (ScreenInfo *scr)
             if( (wm_state == IconicState) || (attr.map_state != IsUnmapped))
 			{
 				LOCAL_DEBUG_OUT( "adding window %lX", children[i] );	  
-                AddWindow( children[i] );
+                AddWindow( children[i], False );
 			}
 		}
 
@@ -641,21 +666,12 @@ Done (Bool restart, char *command )
 		already_dead = True ;
 	}
 
-#ifdef XSHMIMAGE
-	/* may not need to do that as server may still have some of the shared 
-	 * memory and work in it */
-	flush_shm_cache();
-#endif
-
-LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?command:"");
-
     /* lets duplicate the string so we don't accidental;y delete it while closing self down */
     if( restart )
 	{
+		int my_name_len = strlen(MyName);
 		if( command ) 
 		{
-			int my_name_len = strlen(MyName);
-
 			if( strncmp(command, MyName, my_name_len )==0 )
 				restart_self = (command[my_name_len] == ' '|| command[my_name_len] == '\0');
 			local_command = mystrdup(command);
@@ -664,7 +680,37 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
         	local_command = mystrdup(MyName);
 			restart_self = True ;
 	    }
+		if (!is_executable_in_path(local_command))
+		{
+			if (!restart_self || MyArgs.saved_argv[0] == NULL)
+			{
+				show_error("Cannot restart with command \"%s\" - application is not in PATH!", local_command);
+				return;
+			}
+			free(local_command);
+			if (command)
+			{
+				local_command = safemalloc(strlen(command) + 1+ strlen(MyArgs.saved_argv[0])+1);
+				sprintf( local_command, "%s %s", MyArgs.saved_argv[0], command + my_name_len);
+			}else
+				local_command = mystrdup(MyArgs.saved_argv[0]);
+		}
 	}
+
+
+#ifdef XSHMIMAGE
+	/* may not need to do that as server may still have some of the shared 
+	 * memory and work in it */
+	flush_shm_cache();
+#endif
+
+LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?command:"");
+
+	XSelectInput( dpy, Scr.Root, 0 );	   
+	SendPacket (-1, M_SHUTDOWN, 0);
+	FlushAllQueues(); 
+	sleep_a_millisec(1000);
+
 	LOCAL_DEBUG_OUT( "local_command = \"%s\", restart_self = %s", local_command, restart_self?"Yes":"No"); 
     set_flags( AfterStepState, ASS_Shutdown );
     if( restart )
@@ -698,32 +744,43 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
 	if (restart)
 	{
 		set_flags( MyArgs.flags, ASS_Restarting );
-        spawn_child( local_command, -1, restart_screen,
+        spawn_child( local_command, -1, restart_screen, original_DISPLAY_string,
                      None, C_NO_CONTEXT, False, restart_self, NULL );
     } else
 	{
+
 	    XCloseDisplay (dpy);
 		dpy = NULL ;
 
 		/* freeing up memory */
+		DestroyPendingFunctionsQueue();
+		DestroyCategories();
+
+	    cleanup_default_balloons();
+		destroy_asdatabase();
     	destroy_assession( Session );
 		destroy_asenvironment( &Environment );
-    	InitDatabase(True);
-    	free_func_hash ();
     	/* pixmap references */
     	build_xpm_colormap (NULL);
 
-        restack_window_list(INVALID_DESK, True);
+		free_scratch_ids_vector();
+		free_scratch_layers_vector();		
         clientprops_cleanup ();
         wmprops_cleanup ();
+
         free_func_hash();
+		flush_keyword_ids();
         purge_asimage_registry();
+	
 		asxml_var_cleanup();
 		custom_color_cleanup(); 
+
+        free_as_app_args();
+		free( ASDefaultScr );
+
 		flush_default_asstorage();
-        flush_ashash_memory_pool();
         flush_asbidirlist_memory_pool();
-        free( MyArgs.saved_argv );
+        flush_ashash_memory_pool();
 #ifdef DEBUG_ALLOCS
         print_unfreed_mem ();
 #endif /*DEBUG_ALLOCS */
