@@ -182,6 +182,10 @@ typedef struct ASWindow
 #define ASWF_FirstCornerFollowsTbarSize	(0x01<<3)
 #define ASWF_LastCornerFollowsTbarSize	(0x01<<6)
 	ASFlagType internal_flags ;
+  
+  /* we use that to avoid excessive refreshes when property is updated with exact same contents */
+	XWMHints 	saved_wm_hints ; 
+  	XSizeHints 	saved_wm_normal_hints;
 }ASWindow;
 
 typedef struct ASLayer
@@ -245,6 +249,9 @@ typedef struct ASWindowList
      *          some other window. To do that we substitute focused pointer to new window
      *          right before UngrabEm. In this case focused != ( ungrabbed == hilited )
      */
+
+	ASVector	*stacking_order ; 		/* array of pointers to ASWindow structures */
+	
 }ASWindowList;
 
 /* Mirror Note :
@@ -296,8 +303,11 @@ typedef struct queue_buff_struct
 {
   struct queue_buff_struct *next;
   unsigned char            *data;
-  int                       size;
-  int                       done;
+  short                     size;
+  short                     done;
+  short 					prealloced_idx ; 
+  short 					unused;
+  /* 16 bytes total */
 }queue_buff_struct;
 
 typedef struct module_ibuf_t
@@ -410,6 +420,7 @@ ASWindow *bookmark2ASWindow( const char *bookmark );
 ASWindow *pattern2ASWindow( const char *pattern );
 ASWindow *complex_pattern2ASWindow( char *pattern );
 ASLayer *get_aslayer( int layer, ASWindowList *list );
+ASWindow *find_group_lead_aswindow( Window id );
 void tie_aswindow( ASWindow *t );
 void untie_aswindow( ASWindow *t );
 void add_aswindow_to_layer( ASWindow *asw, int layer );
@@ -417,7 +428,15 @@ void remove_aswindow_from_layer( ASWindow *asw, int layer );
 Bool enlist_aswindow( ASWindow *t );
 void delist_aswindow( ASWindow *t );
 void save_aswindow_list( ASWindowList *list, char *file );
-void restack_window_list( int desk, Bool send_msg_only );
+ASWindow* find_topmost_client( int desk, int root_x, int root_y );
+void free_scratch_layers_vector();
+void free_scratch_ids_vector();
+void update_stacking_order();
+void restack_window_list( int desk);
+void send_stacking_order( int desk );
+void apply_stacking_order( int desk );
+
+
 Bool is_window_obscured (ASWindow * above, ASWindow * below);
 void restack_window( ASWindow *t, Window sibling_window, int stack_mode );
 
@@ -440,7 +459,10 @@ MenuData *make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order,
 
 void hide_focus();
 Bool focus_window( ASWindow *asw, Window w );
-Bool focus_aswindow( ASWindow *asw );
+void autoraise_window( ASWindow *asw );
+#define FOCUS_ASW_CAN_AUTORAISE		False
+#define FOCUS_ASW_DONT_AUTORAISE	True
+Bool focus_aswindow( ASWindow *asw, Bool suppress_autoraise );
 Bool focus_active_window();
 void focus_next_aswindow( ASWindow *asw );     /* should be called when window is unmapped or destroyed */
 void focus_prev_aswindow( ASWindow *asw );     /* should be called when window is unmapped or destroyed */
@@ -463,7 +485,7 @@ void on_icon_changed( ASWindow *asw );
 void on_window_title_changed( ASWindow *asw, Bool update_display );
 void on_window_hints_changed( ASWindow *asw );
 void on_window_opacity_changed( ASWindow *asw );
-void on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured );
+void on_window_status_changed( ASWindow *asw, Bool reconfigured );
 void on_window_hilite_changed( ASWindow *asw, Bool focused );
 void on_window_pressure_changed( ASWindow *asw, int pressed_context );
 
@@ -696,14 +718,15 @@ void release_all_old_background( Bool forget );
 
 
 /******************************* placement.c *******************************/
-ASGrid* make_desktop_grid(int desk, int min_layer, Bool frame_only, int vx, int vy, ASWindow *target );
-Bool place_aswindow( ASWindow *asw );
-void apply_aswindow_move(struct ASMoveResizeData *data);
-void apply_aswindow_moveresize(struct ASMoveResizeData *data);
-void complete_aswindow_move(struct ASMoveResizeData *data, Bool cancelled);
-void complete_aswindow_moveresize(struct ASMoveResizeData *data, Bool cancelled);
-void enforce_avoid_cover(ASWindow *asw );
-void obey_avoid_cover(ASWindow *asw, ASStatusHints *tmp_status, XRectangle *tmp_anchor );
+ASGrid* make_desktop_grid (int desk, int min_layer, Bool frame_only, int vx, int vy, ASWindow *target);
+void setup_aswindow_moveresize (ASWindow *asw,  struct ASMoveResizeData *mvrdata);
+Bool place_aswindow (ASWindow *asw);
+void apply_aswindow_move (struct ASMoveResizeData *data);
+void apply_aswindow_moveresize (struct ASMoveResizeData *data);
+void complete_aswindow_move (struct ASMoveResizeData *data, Bool cancelled);
+void complete_aswindow_moveresize (struct ASMoveResizeData *data, Bool cancelled);
+void enforce_avoid_cover (ASWindow *asw);
+void obey_avoid_cover (ASWindow *asw, ASStatusHints *tmp_status, XRectangle *tmp_anchor, int max_layer);
 
 
 /******************************* theme.c ***********************************/
@@ -732,7 +755,7 @@ void save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert );
 void moveresize_aswindow_wm( ASWindow *asw, int x, int y, unsigned int width, unsigned int height, Bool save_anchor );
 
 void on_window_anchor_changed( ASWindow *asw );
-void validate_window_anchor( ASWindow *asw, XRectangle *new_anchor );
+void validate_window_anchor (ASWindow *asw, XRectangle *new_anchor, Bool initial_placement);
 
 
 Bool init_aswindow_status( ASWindow *t, ASStatusHints *status );
