@@ -338,19 +338,62 @@ DigestEvent( ASEvent *event )
 {
     register int i ;
     setup_asevent_from_xevent( event );
+    event->client = NULL;
+    SHOW_EVENT_TRACE(event);
     /* in housekeeping mode we handle pointer events only as applied to root window ! */
     if( Scr.moveresize_in_progress && (event->eclass & ASE_POINTER_EVENTS) != 0)
     {
 	    event->context = C_ROOT ;
     	event->widget = Scr.RootCanvas ;
-        event->client = NULL;
         /* we have to do this at all times !!!! */
         if( event->x.type == ButtonRelease && Scr.Windows->pressed )
             release_pressure();
     }else
 	{
-		if( event->w == Scr.Root || 
-			event->w == Scr.ServiceWin ||
+	    XButtonEvent *xbtn = &(event->x.xbutton);
+
+		if( event->w == Scr.Root )
+		{
+			event->context = C_ROOT ;
+#ifndef NO_VIRTUAL
+			if((event->eclass & ASE_MousePressEvent) != 0 && xbtn->subwindow != None )
+			{
+				LOCAL_DEBUG_OUT( "subwindow = %lX", event->x.xbutton.subwindow );
+	    		for( i = 0 ; i < PAN_FRAME_SIDES ; i++ )
+	    		{
+	        		LOCAL_DEBUG_OUT("checking panframe %d, mapped %d", i, Scr.PanFrame[i].isMapped );
+	        		if( Scr.PanFrame[i].isMapped && xbtn->subwindow == Scr.PanFrame[i].win)
+        			{/* we should try and pass through this click onto any client that 
+						maybe under the pan frames */
+						LOCAL_DEBUG_OUT( "looking for client at %+d%+d", xbtn->x_root, xbtn->y_root ); 
+						if( (event->client = find_topmost_client( Scr.CurrentDesk, xbtn->x_root, xbtn->y_root )) != NULL )
+						{
+					    	LOCAL_DEBUG_OUT("underlying new client %p", event->client );
+							ASCanvas *cc = event->client->client_canvas ; 
+							if( cc && cc->root_x <= xbtn->x_root && cc->root_y <= xbtn->y_root &&
+								cc->root_x+cc->width > xbtn->x_root && cc->root_y+cc->height > xbtn->y_root )
+							{
+								event->w = xbtn->window = event->client->w ; 
+								xbtn->subwindow = None ; 
+						        XSendEvent (dpy, event->client->w, False, 
+											(xbtn->type == ButtonPress)?ButtonPressMask:ButtonReleaseMask, &(event->x));
+								xbtn->subwindow = event->client->w ; 
+								xbtn->window = Scr.Root ;
+								/* xbtn->window should still be Root, so we can proxy it down to the app */
+							}else
+							{
+								event->w = xbtn->window = event->client->frame ; 
+								xbtn->subwindow = None ;
+							}
+					    	LOCAL_DEBUG_OUT("new event window %lX", event->w );
+							event->context = C_NO_CONTEXT ;						
+						}
+						break;
+					}
+				}
+			}
+#endif
+		}else if( event->w == Scr.ServiceWin ||
 			event->w == Scr.SizeWindow )
 		{
 	    	event->context = C_ROOT ; 
@@ -361,7 +404,7 @@ DigestEvent( ASEvent *event )
 		{	
     		event->widget = Scr.RootCanvas ;
 			event->client = NULL ;
-		}else
+		}else if(event->client == NULL )
 		{
 			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) != NULL  )
 			{	
@@ -722,9 +765,12 @@ HandleFocusIn ( ASEvent *event )
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         ChangeWarpingFocus( event->client );
 
+	LOCAL_DEBUG_OUT( "focused = %p, this event for %p", Scr.Windows->focused, event->client );
     if( Scr.Windows->focused != event->client )
+	{
+		LOCAL_DEBUG_OUT( "CHANGE Scr.Windows->focused from %p to NULL", Scr.Windows->focused );
         Scr.Windows->focused = NULL ;
-
+	}
     if( event->client == NULL && get_flags(AfterStepState, ASS_HousekeepingMode))
         return;
     /* note that hilite_aswindow changes value of Scr.Hilite!!! */
@@ -815,6 +861,61 @@ update_transp_iter_func(void *data, void *aux_data)
     return True;
 }
 
+static Bool 
+check_wm_hints_changed( ASWindow *asw )
+{
+	unsigned char *ptr = (unsigned char*)&(asw->saved_wm_hints) ; 
+	XWMHints *tmp = XGetWMHints (dpy, asw->w);
+	Bool changed = False ; 
+	if( tmp == NULL )
+	{
+		int i ;
+		for( i = 0 ; i < sizeof(XWMHints); ++i ) 
+			if( ptr[i] != 0 ) 
+				return True ;
+	}else
+	{
+		changed = (memcmp( tmp, ptr, sizeof(XWMHints) ) != 0);
+		XFree( tmp );
+	}
+	return changed ;
+}
+
+static Bool 
+check_wm_normal_hints_changed( ASWindow *asw )
+{
+	unsigned char *ptr = (unsigned char*)&(asw->saved_wm_normal_hints) ; 
+	XSizeHints *tmp = XAllocSizeHints();
+	unsigned char *ptr2 = (unsigned char *)tmp ;
+	Bool changed = False ; 
+	long unused ; 
+	int i ;
+
+	if( XGetWMNormalHints (dpy, asw->w, tmp, &unused) == 0 )
+	{
+		for( i = 0 ; i < sizeof(XSizeHints); ++i ) 
+			if( ptr[i] != 0 ) 
+			{	changed = True ; break ; }
+	}else
+		for( i = 0 ; i < sizeof(XSizeHints); ++i ) 
+			if( ptr[i] != ptr2[i] ) 
+			{	changed = True ; break ; }
+
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
+	if( changed ) 
+	{
+		LOCAL_DEBUG_OUT( "normal hints differ at offset %d", i ); 
+		LOCAL_DEBUG_OUT( "Old hints : %s","" ); 
+		print_wm_normal_hints  ( NULL, NULL, (XSizeHints*)ptr );
+		LOCAL_DEBUG_OUT( "New hints : %s","" ); 
+		print_wm_normal_hints  ( NULL, NULL, tmp );
+	}
+#endif	
+	XFree( tmp );
+	return changed ;
+}
+
+
 void
 HandlePropertyNotify (ASEvent *event)
 {
@@ -910,6 +1011,20 @@ HandlePropertyNotify (ASEvent *event)
 		if( old_name )
 			free( old_name );
     	LOCAL_DEBUG_OUT( "hints flags = %lX, ShortLived ? %lX ", asw->hints->flags, ASWIN_HFLAGS( asw, AS_ShortLived ) );
+#if (defined(LOCAL_DEBUG)||defined(DEBUG)) && defined(DEBUG_ALLOCS)
+		{
+			static time_t old_t = 0 ; 
+			time_t t = time(NULL);
+			if( old_t < t ) 
+			{
+				char fname[256] ; 
+				sprintf( fname, "afterstep.allocs.name_change.%lu.log", t ); 
+//		    	spool_unfreed_mem( fname, NULL );
+				old_t = t ;
+			}
+		}
+#endif
+
 	/* otherwise we should check if this is the status property that we change ourselves : */
     }else if( atom == XA_WM_COMMAND || atom == XA_WM_CLIENT_MACHINE )
 	{
@@ -918,6 +1033,22 @@ HandlePropertyNotify (ASEvent *event)
 	{	
 	 	if( event->context == C_WINDOW && event->w != asw->frame ) 
 			on_window_opacity_changed( asw );
+	}else if( atom == XA_WM_HINTS )
+	{
+		if( check_wm_hints_changed( asw ) ) 
+	        on_window_hints_changed( asw );
+		else
+		{
+    		LOCAL_DEBUG_OUT( "ignoring WM_HINTS change - data is the same%s","" );
+		}		
+	}else if( atom == XA_WM_NORMAL_HINTS )
+	{
+		if( check_wm_normal_hints_changed( asw ) ) 
+	        on_window_hints_changed( asw );
+		else
+		{
+    		LOCAL_DEBUG_OUT( "ignoring WM_NORMAL_HINTS change - data is the same%s","" );
+		}		
 	}else if( NeedToTrackPropChanges(atom) )
         on_window_hints_changed( asw );
 
@@ -925,6 +1056,52 @@ HandlePropertyNotify (ASEvent *event)
 }
 
 
+#ifdef NAME_CHANGE_TEST
+/* our status */
+ASFlagType AfterStepState = 0; /* default status */
+
+/* Config : */
+ASVector     *Modules       = NULL;
+int           Module_fd     = 0;
+int           Module_npipes = 8;
+
+ASBalloonState *MenuBalloons = NULL ; 
+ASBalloonState *TitlebarBalloons = NULL ; 
+
+void
+Done (Bool restart, char *command )
+{
+	    exit(0);
+}
+
+int
+main(int argc, char **argv)
+{
+	Window w ;
+	int i = 0;
+	char window_name[512] ; 
+
+    InitMyApp ("test_title_change", argc, argv, NULL, NULL, 0 );
+	
+	ConnectX(ASDefaultScr, 0);
+	
+	w =  create_visual_window( Scr.asv, Scr.Root, 0, 0, 300, 300, 0, InputOutput, 0, NULL);
+	
+	sprintf( window_name, "test title change - iter %d", i ) ;
+    set_client_names( w, window_name, window_name, "test", "test_title_change" );
+	
+	XSetWindowBackground(dpy, w, 0 );
+	XMapWindow( dpy, w );
+	while(1)
+	{
+		ASSync(False);
+		sleep_a_millisec(100);
+		sprintf( window_name, "test title change - iter %d", ++i ) ;
+	    set_client_names( w, window_name, window_name, "test", "test_title_change" );
+	}	
+	return 0;
+}
+#endif
 /***********************************************************************
  *
  *  Procedure:
@@ -1022,13 +1199,22 @@ HandleDestroyNotify (ASEvent *event )
  *  Procedure:
  *	HandleMapRequest - MapRequest event handler
  ************************************************************************/
+void
+delayed_add_window( void* vdata )
+{
+	if( window2ASWindow((Window)vdata) == NULL )
+		AddWindow ((Window)vdata, True);
+}
+
 void
 HandleMapRequest (ASEvent *event )
 {
     /* If the window has never been mapped before ... */
     if (event->client == NULL)
-    {
-        if( (event->client = AddWindow (event->w, True)) == NULL )
+    {/* lets delay handling map request in case client needs time to update its properties */
+		timer_new (200, delayed_add_window, (void*)event->w);	
+
+/*        if( (event->client = AddWindow (event->w, True)) == NULL ) */
             return;
     }else /* If no hints, or currently an icon, just "deiconify" */
         set_window_wm_state( event->client, False, True );
@@ -1368,6 +1554,7 @@ HandleConfigureRequest ( ASEvent *event )
     XConfigureRequestEvent *cre = &(event->x.xconfigurerequest);
     ASWindow *asw = event->client ;
     XWindowChanges xwc;
+    unsigned long xwcm;
 
     /*
 	 * According to the July 27, 1988 ICCCM draft, we should ignore size and
@@ -1380,7 +1567,6 @@ HandleConfigureRequest ( ASEvent *event )
 	cre->x, cre->y );
     if (asw == NULL)
 	{
-        unsigned long xwcm;
         xwcm = cre->value_mask & (CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
 		if( xwcm == 0 ) 
 		{
@@ -1399,17 +1585,49 @@ HandleConfigureRequest ( ASEvent *event )
 		return;
 	}
 
+	if( asw->icon_canvas && event->w == asw->icon_canvas->w ) 
+	{ /* we really should ignore that ! - let's see how it will play out */
+	  /* we may need to add code to iconbox to handle custom icon geometry for client icons */
+        xwcm = cre->value_mask & (CWWidth | CWHeight);
+		if( xwcm == 0 ) 
+		{
+			LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for iconic window %lX - no supported changes detected.", (unsigned long)event->w );
+		}else
+		{
+			xwc.width = cre->width;
+			xwc.height = cre->height;
+			LOCAL_DEBUG_OUT( "Configuring iconic window %lX to %dx%d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, xwcm );
+        	XConfigureWindow (dpy, event->w, xwcm, &xwc);
+			ASSync( False);
+		}
+		return;
+	}
+	
+	if( event->w != asw->w ) 
+	{
+		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for window %lX. Not a client or client's icon!", (unsigned long)event->w );
+		return;
+	}
+
+    if (cre->value_mask & CWStackMode )
+	{
+		if( !ASWIN_HFLAGS(asw, AS_IgnoreRestackRequest ) )
+		{
+	        restack_window( asw, (cre->value_mask & CWSibling)?cre->above:None, cre->detail );
+		}else
+		{
+			LOCAL_DEBUG_OUT( "Ignoring Stacking order Request for client %p as required by hints", asw );
+		}
+	}
+
 	if( ( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) && !get_flags(cre->value_mask, CWWidth|CWHeight))||
 		ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
 	{	
 		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
-		send_canvas_configure_notify(asw->frame_canvas, asw->client_canvas);
+		SendConfigureNotify(asw);
 		return;
 	}
 
-    if (cre->value_mask & CWStackMode)
-        restack_window( asw, (cre->value_mask & CWSibling)?cre->above:None, cre->detail );
-
     /* check_aswindow_shaped( asw ); */
 
     /* for restoring */
@@ -1436,11 +1654,13 @@ HandleConfigureRequest ( ASEvent *event )
             	new_anchor.y = make_anchor_pos ( asw->status, cre->y, new_anchor.height, Scr.Vy, grav_y, Scr.VyMax+Scr.MyDisplayHeight );
 		}
 LOCAL_DEBUG_OUT( "old anchor(%dx%d%+d%+d), new_anchor(%dx%d%+d%+d)", asw->anchor.width, asw->anchor.height, asw->anchor.x, asw->anchor.y, new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
-        validate_window_anchor( asw, &new_anchor );
+        validate_window_anchor( asw, &new_anchor, False );
 LOCAL_DEBUG_OUT( "validated_anchor(%dx%d%+d%+d)", new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
         asw->anchor = new_anchor ;
         on_window_anchor_changed( asw );
 		enforce_avoid_cover( asw );
+        if( (cre->value_mask&(CWWidth|CWHeight)) == 0 )
+			SendConfigureNotify(asw);		
     }
 }
 
@@ -1591,12 +1811,13 @@ afterstep_wait_pipes_input(int timeout_sec)
         i = MIN(MODULES_NUM,Module_npipes) ;
         /* Check for module input. */
         while( --i >= 0 )
-        {
-            Bool has_input = FD_ISSET (list[i].fd, &in_fdset);
-            Bool has_output = FD_ISSET (list[i].fd, &out_fdset);
-            if( has_input || has_output )
-                HandleModuleInOut(i, has_input, has_output);
-        }
+			if( list[i].fd > 0 )
+	        {
+        	    Bool has_input = FD_ISSET (list[i].fd, &in_fdset);
+    	        Bool has_output = FD_ISSET (list[i].fd, &out_fdset);
+            	if( has_input || has_output )
+                	HandleModuleInOut(i, has_input, has_output);
+	        }
 	}
 
 	/* handle timeout events */
